
#include "main.h"

unsigned char NRF_TX_ADDRESS[NRF_TX_ADDR_WIDTH]={0x34,0x43,0x10,0x10,0x01}; //发送地址
unsigned char NRF_RX_ADDRESS[NRF_RX_ADDR_WIDTH]={0x34,0x43,0x10,0x10,0x01}; //发送地址

unsigned char nrf_write_reg(unsigned char reg,unsigned char value)
{
    unsigned char status;

    NRF24L01_CS_L();
    status = spi2_read_write(NRF_WRITE_REG|reg);
    spi2_read_write(value);
    NRF24L01_CS_H();
    return(status);
}

unsigned char nrf_read_reg(unsigned char reg)
{
    unsigned char reg_val;

    NRF24L01_CS_L();
    spi2_read_write(NRF_READ_REG|reg);
    reg_val=spi2_read_write(0XFF);
    NRF24L01_CS_H();
    return(reg_val);
}

unsigned char nrf_write_buf(unsigned char reg, unsigned char *pBuf, unsigned char len)
{
    unsigned char status;

    NRF24L01_CS_L();
    status = spi2_read_write( NRF_WRITE_REG | reg );
    for(int i=0; i<len; i++)
    {
        spi2_read_write(*pBuf++);
    }
    NRF24L01_CS_H();
    return status;
}

unsigned char nrf_read_buf(unsigned char reg,unsigned char *pBuf,unsigned char len)
{
    unsigned char status;

    NRF24L01_CS_L();
    status=spi2_read_write(NRF_READ_REG|reg);
    for(int i=0;i<len;i++)
    {
        pBuf[i]=spi2_read_write(0XFF);
    }
    NRF24L01_CS_H();
    return status;
}


unsigned char nrf_check( void )
{
    uint8_t buf[5]={ 0XA5, 0XA5, 0XA5, 0XA5, 0XA5 };
    uint8_t read_buf[5]={ 0 };

    nrf_write_buf(NRF_TX_ADDR,buf,5);
    nrf_read_buf(NRF_TX_ADDR,read_buf,5);
    for(int i=0;i<5;i++)
    {
        if(buf[i]!=read_buf[i])
        {
            return 1;
        }
    }
    return 0;
}

void nrf_clear_tx_fifo ( void )
{
    NRF24L01_CS_L();
    spi2_read_write( NRF_FLUSH_TX );
    NRF24L01_CS_H();
}

void nrf_clear_rx_fifo( void )
{
    NRF24L01_CS_L( );       //片选
    spi2_read_write(NRF_FLUSH_RX ); //清RX FIFO命令
    NRF24L01_CS_H( );       //取消片选
}

void nrf_init(void) {
    GPIO_InitTypeDef GPIO_InitStructure;
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB, ENABLE);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8; // CE
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12; // CS
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    NRF24L01_CE_L();
    NRF24L01_CS_L();

    while(nrf_check()){;}
    nrf_write_reg(NRF_SETUP_AW, 0x03);//地址长度
    nrf_write_buf(NRF_TX_ADDR, NRF_TX_ADDRESS, NRF_TX_ADDR_WIDTH);//发送地址
    nrf_write_buf(NRF_RX_ADDR_P0, NRF_RX_ADDRESS, NRF_RX_ADDR_WIDTH);//接收地址
    nrf_write_reg(NRF_EN_AA, 0x01);//使能通道0自动应答
    nrf_write_reg(NRF_EN_RXADDR, 0x01);//使能通道0接收
    nrf_write_reg(NRF_RX_PW_P0, 32);//设置负载长度
    nrf_write_reg(NRF_DYNPD,0x01);//使能通道0动态数据长度
    nrf_write_reg(NRF_FEATURE,0x06);//使能动态负载
    nrf_write_reg(NRF_SETUP_RETR,0x14);//不重发，重发间隔1ms
    nrf_write_reg(NRF_RF_SETUP, 0x07);//1M/7db
    nrf_write_reg(NRF_RF_CH, 0x40); //选择射频信道
    nrf_write_reg(NRF_CONFIG, 0x7b);//接收模式
    NRF24L01_CE_H();
}


// 函数：分块发送2048字节数据
void nrf_send_large_packet(unsigned char *data, unsigned int length)
{
    unsigned int chunkSize = 255;
    unsigned int offset = 0;

    while (length > 0)
    {
        unsigned int sendSize = (length > chunkSize) ? chunkSize : length;
        nrf_write_buf(NRF_W_ACK_PLOAD, data + offset, sendSize);
        offset += sendSize;
        length -= sendSize;
    }
}


void nrf_receive_large_packet(unsigned char *data, unsigned int length)
{
    unsigned int chunkSize = 255;
    unsigned int offset = 0;

    while (length > 0)
    {
        unsigned int receiveSize = (length > chunkSize) ? chunkSize : length;
        nrf_read_buf(NRF_RD_RX_PLOAD, data + offset, receiveSize);
        offset += receiveSize;
        length -= receiveSize;
    }
}


uint8_t nrf_check_presence(void)
{
    uint8_t nrf_status;
    uint8_t nrf_config;

    nrf_config = nrf_read_reg(NRF_CONFIG);

    if (nrf_config != 0x08) {
        nrf_status = nrf_read_reg(NRF_STATUS);
        if ((nrf_status & (NRF_STATUS_RX_DR | NRF_STATUS_TX_DS | NRF_STATUS_MAX_RT)) == 0) {
            return 1; // NRF24L01 is present and responsive
        }
    }

    return 0; // NRF24L01 not detected or not responsive
}


